@@ -51,7 +51,7 @@ QAction *QgsLayerTreeViewDefaultActions::actionShowInOverview( QObject *parent )
   if ( !node )
     return nullptr;
 
-  QAction *a = new QAction( QgsApplication::getThemeIcon( QStringLiteral( "/mActionInOverview.svg" ) ), tr( "&Show in Overview" ), parent );
+  QAction *a = new QAction( QgsApplication::getThemeIcon( QStringLiteral( "/mActionInOverview.svg" ) ), tr( "Show in &Overview" ), parent );
   connect( a, &QAction::triggered, this, &QgsLayerTreeViewDefaultActions::showInOverview );
   a->setCheckable( true );
   a->setChecked( node->customProperty( QStringLiteral( "overview" ), 0 ).toInt() );
@@ -81,7 +81,7 @@ QAction *QgsLayerTreeViewDefaultActions::actionShowFeatureCount( QObject *parent
   if ( !node )
     return nullptr;
 
-  QAction *a = new QAction( tr( "Show Feature Count" ), parent );
+  QAction *a = new QAction( tr( "Show Feature &Count" ), parent );
   connect( a, &QAction::triggered, this, &QgsLayerTreeViewDefaultActions::showFeatureCount );
   a->setCheckable( true );
   a->setChecked( node->customProperty( QStringLiteral( "showFeatureCount" ), 0 ).toInt() );
@@ -93,14 +93,25 @@ QAction *QgsLayerTreeViewDefaultActions::actionZoomToLayer( QgsMapCanvas *canvas
   QAction *a = new QAction( QgsApplication::getThemeIcon( QStringLiteral( "/mActionZoomToLayer.svg" ) ),
                             tr( "&Zoom to Layer" ), parent );
   a->setData( QVariant::fromValue( reinterpret_cast<void *>( canvas ) ) );
+  Q_NOWARN_DEPRECATED_PUSH
   connect( a, &QAction::triggered, this, static_cast<void ( QgsLayerTreeViewDefaultActions::* )()>( &QgsLayerTreeViewDefaultActions::zoomToLayer ) );
+  Q_NOWARN_DEPRECATED_POP
+  return a;
+}
+
+QAction *QgsLayerTreeViewDefaultActions::actionZoomToLayers( QgsMapCanvas *canvas, QObject *parent )
+{
+  QAction *a = new QAction( QgsApplication::getThemeIcon( QStringLiteral( "/mActionZoomToLayer.svg" ) ),
+                            tr( "&Zoom to Layer(s)" ), parent );
+  a->setData( QVariant::fromValue( canvas ) );
+  connect( a, &QAction::triggered, this, static_cast<void ( QgsLayerTreeViewDefaultActions::* )()>( &QgsLayerTreeViewDefaultActions::zoomToLayers ) );
   return a;
 }
 
 QAction *QgsLayerTreeViewDefaultActions::actionZoomToSelection( QgsMapCanvas *canvas, QObject *parent )
 {
   QAction *a = new QAction( QgsApplication::getThemeIcon( QStringLiteral( "/mActionZoomToSelected.svg" ) ),
-                            tr( "&Zoom to Selection" ), parent );
+                            tr( "Zoom to &Selection" ), parent );
   a->setData( QVariant::fromValue( reinterpret_cast<void *>( canvas ) ) );
   connect( a, &QAction::triggered, this, static_cast<void ( QgsLayerTreeViewDefaultActions::* )()>( &QgsLayerTreeViewDefaultActions::zoomToSelection ) );
   return a;
@@ -126,7 +137,7 @@ QAction *QgsLayerTreeViewDefaultActions::actionMakeTopLevel( QObject *parent )
 
 QAction *QgsLayerTreeViewDefaultActions::actionMoveOutOfGroup( QObject *parent )
 {
-  QAction *a = new QAction( tr( "Move Out of &Group" ), parent );
+  QAction *a = new QAction( tr( "Move O&ut of Group" ), parent );
   connect( a, &QAction::triggered, this, &QgsLayerTreeViewDefaultActions::moveOutOfGroup );
   return a;
 }
@@ -158,7 +169,7 @@ QAction *QgsLayerTreeViewDefaultActions::actionMutuallyExclusiveGroup( QObject *
   if ( !node || !QgsLayerTree::isGroup( node ) )
     return nullptr;
 
-  QAction *a = new QAction( tr( "Mutually Exclusive Group" ), parent );
+  QAction *a = new QAction( tr( "&Mutually Exclusive Group" ), parent );
   a->setCheckable( true );
   a->setChecked( QgsLayerTree::toGroup( node )->isMutuallyExclusive() );
   connect( a, &QAction::triggered, this, &QgsLayerTreeViewDefaultActions::mutuallyExclusiveGroup );
@@ -198,7 +209,7 @@ QAction *QgsLayerTreeViewDefaultActions::actionCheckAndAllParents( QObject *pare
   QgsLayerTreeNode *node = mView->currentNode();
   if ( !node || !QgsLayerTree::isLayer( node ) || node->isVisible() )
     return nullptr;
-  QAction *a = new QAction( tr( "Check and All its Parents" ), parent );
+  QAction *a = new QAction( tr( "Chec&k and All its Parents" ), parent );
   connect( a, &QAction::triggered, this, &QgsLayerTreeViewDefaultActions::checkAndAllParents );
   return a;
 }
@@ -229,7 +240,8 @@ void QgsLayerTreeViewDefaultActions::checkAndAllParents()
 
 void QgsLayerTreeViewDefaultActions::addGroup()
 {
-  if ( mView->selectedNodes( true ).count() >= 2 )
+  int nodeCount = mView->selectedNodes( true ).count();
+  if ( nodeCount > 1 || ( nodeCount == 1 && mView->currentLayer() ) )
   {
     groupSelected();
     return;
@@ -239,7 +251,7 @@ void QgsLayerTreeViewDefaultActions::addGroup()
     group = mView->layerTreeModel()->rootGroup();
 
   QgsLayerTreeGroup *newGroup = group->addGroup( uniqueGroupName( group ) );
-  mView->edit( mView->layerTreeModel()->node2index( newGroup ) );
+  mView->edit( mView->node2index( newGroup ) );
 }
 
 void QgsLayerTreeViewDefaultActions::removeGroupOrLayer()
@@ -280,25 +292,34 @@ void QgsLayerTreeViewDefaultActions::showFeatureCount()
     l->setCustomProperty( QStringLiteral( "showFeatureCount" ), newValue ? 0 : 1 );
 }
 
-
 void QgsLayerTreeViewDefaultActions::zoomToLayer( QgsMapCanvas *canvas )
 {
   QgsMapLayer *layer = mView->currentLayer();
   if ( !layer )
     return;
 
-  QList<QgsMapLayer *> layers;
-  layers << layer;
+  const QList<QgsMapLayer *> layers { layer };
+  zoomToLayers( canvas, layers );
+}
+
+void QgsLayerTreeViewDefaultActions::zoomToLayers( QgsMapCanvas *canvas )
+{
+  const QList<QgsMapLayer *> layers = mView->selectedLayers();
+
   zoomToLayers( canvas, layers );
 }
 
 void QgsLayerTreeViewDefaultActions::zoomToSelection( QgsMapCanvas *canvas )
 {
-  QgsVectorLayer *layer = qobject_cast<QgsVectorLayer *>( mView->currentLayer() );
-  if ( !layer )
-    return;
+  QgsVectorLayer *layer = qobject_cast< QgsVectorLayer * >( mView->currentLayer() );
+
+  const QList<QgsMapLayer *> layers = mView->selectedLayers();
+
+  if ( layers.size() > 1 )
+    canvas->zoomToSelected( layers );
+  else if ( layers.size() <= 1 && layer )
+    canvas->zoomToSelected( layer );
 
-  canvas->zoomToSelected( layer );
 }
 
 void QgsLayerTreeViewDefaultActions::zoomToGroup( QgsMapCanvas *canvas )
@@ -308,8 +329,8 @@ void QgsLayerTreeViewDefaultActions::zoomToGroup( QgsMapCanvas *canvas )
     return;
 
   QList<QgsMapLayer *> layers;
-  const auto constFindLayerIds = groupNode->findLayerIds();
-  for ( const QString &layerId : constFindLayerIds )
+  const QStringList findLayerIds = groupNode->findLayerIds();
+  for ( const QString &layerId : findLayerIds )
     layers << QgsProject::instance()->mapLayer( layerId );
 
   zoomToLayers( canvas, layers );
@@ -317,18 +338,25 @@ void QgsLayerTreeViewDefaultActions::zoomToGroup( QgsMapCanvas *canvas )
 
 void QgsLayerTreeViewDefaultActions::zoomToLayer()
 {
+  Q_NOWARN_DEPRECATED_PUSH
   QAction *s = qobject_cast<QAction *>( sender() );
   QgsMapCanvas *canvas = reinterpret_cast<QgsMapCanvas *>( s->data().value<void *>() );
-  QApplication::setOverrideCursor( Qt::WaitCursor );
+
   zoomToLayer( canvas );
-  QApplication::restoreOverrideCursor();
+  Q_NOWARN_DEPRECATED_POP
+}
+
+void QgsLayerTreeViewDefaultActions::zoomToLayers()
+{
+  QAction *s = qobject_cast<QAction *>( sender() );
+  QgsMapCanvas *canvas = s->data().value<QgsMapCanvas *>();
+  zoomToLayers( canvas );
 }
 
 void QgsLayerTreeViewDefaultActions::zoomToSelection()
 {
   QAction *s = qobject_cast<QAction *>( sender() );
   QgsMapCanvas *canvas = reinterpret_cast<QgsMapCanvas *>( s->data().value<void *>() );
-  QgsTemporaryCursorOverride waitCursor( Qt::WaitCursor );
   zoomToSelection( canvas );
 }
 
@@ -336,41 +364,51 @@ void QgsLayerTreeViewDefaultActions::zoomToGroup()
 {
   QAction *s = qobject_cast<QAction *>( sender() );
   QgsMapCanvas *canvas = reinterpret_cast<QgsMapCanvas *>( s->data().value<void *>() );
-  QApplication::setOverrideCursor( Qt::WaitCursor );
   zoomToGroup( canvas );
-  QApplication::restoreOverrideCursor();
 }
 
-
 void QgsLayerTreeViewDefaultActions::zoomToLayers( QgsMapCanvas *canvas, const QList<QgsMapLayer *> &layers )
 {
+  QgsTemporaryCursorOverride cursorOverride( Qt::WaitCursor );
+
   QgsRectangle extent;
-  extent.setMinimal();
+  extent.setNull();
 
-  for ( int i = 0; i < layers.size(); ++i )
+  if ( !layers.empty() )
   {
-    QgsMapLayer *layer = layers.at( i );
-    QgsRectangle layerExtent = layer->extent();
-
-    QgsVectorLayer *vLayer = qobject_cast<QgsVectorLayer *>( layer );
-    if ( vLayer )
+    for ( int i = 0; i < layers.size(); ++i )
     {
-      if ( vLayer->geometryType() == QgsWkbTypes::NullGeometry )
+      QgsMapLayer *layer = layers.at( i );
+      QgsRectangle layerExtent = layer->extent();
+
+      QgsVectorLayer *vLayer = qobject_cast<QgsVectorLayer *>( layer );
+      if ( vLayer )
+      {
+        if ( vLayer->geometryType() == Qgis::GeometryType::Null )
+          continue;
+
+        if ( layerExtent.isEmpty() )
+        {
+          vLayer->updateExtents();
+          layerExtent = vLayer->extent();
+        }
+      }
+
+      if ( layerExtent.isNull() )
         continue;
 
-      if ( layerExtent.isEmpty() )
-      {
-        vLayer->updateExtents();
-        layerExtent = vLayer->extent();
-      }
+      //transform extent
+      layerExtent = canvas->mapSettings().layerExtentToOutputExtent( layer, layerExtent );
+
+      extent.combineExtentWith( layerExtent );
     }
+  }
 
-    if ( layerExtent.isNull() )
-      continue;
-
-    //transform extent
-    layerExtent = canvas->mapSettings().layerExtentToOutputExtent( layer, layerExtent );
-
+  // If no layer is selected, use current layer
+  else if ( mView->currentLayer() )
+  {
+    QgsRectangle layerExtent = mView->currentLayer()->extent();
+    layerExtent = canvas->mapSettings().layerExtentToOutputExtent( mView->currentLayer(), layerExtent );
     extent.combineExtentWith( layerExtent );
   }
 
@@ -443,7 +481,7 @@ void QgsLayerTreeViewDefaultActions::moveToTop()
   {
     return a->depth() > b->depth();
   } );
-  for ( QgsLayerTreeNode *n : qgis::as_const( selectedNodes ) )
+  for ( QgsLayerTreeNode *n : std::as_const( selectedNodes ) )
   {
     QgsLayerTreeGroup *parentGroup = qobject_cast<QgsLayerTreeGroup *>( n->parent() );
     QgsLayerTreeNode *clonedNode = n->clone();
@@ -461,7 +499,7 @@ void QgsLayerTreeViewDefaultActions::moveToBottom()
   {
     return a->depth() > b->depth();
   } );
-  for ( QgsLayerTreeNode *n : qgis::as_const( selectedNodes ) )
+  for ( QgsLayerTreeNode *n : std::as_const( selectedNodes ) )
   {
     QgsLayerTreeGroup *parentGroup = qobject_cast<QgsLayerTreeGroup *>( n->parent() );
     QgsLayerTreeNode *clonedNode = n->clone();
@@ -474,7 +512,7 @@ void QgsLayerTreeViewDefaultActions::moveToBottom()
 void QgsLayerTreeViewDefaultActions::groupSelected()
 {
   const QList<QgsLayerTreeNode *> nodes = mView->selectedNodes( true );
-  if ( nodes.count() < 2 || ! QgsLayerTree::isGroup( nodes[0]->parent() ) )
+  if ( nodes.empty() || ! QgsLayerTree::isGroup( nodes[0]->parent() ) )
     return;
 
   QgsLayerTreeGroup *parentGroup = QgsLayerTree::toGroup( nodes[0]->parent() );
@@ -493,8 +531,8 @@ void QgsLayerTreeViewDefaultActions::groupSelected()
       group->removeChildNode( node );
   }
 
-  mView->setCurrentIndex( mView->layerTreeModel()->node2index( newGroup ) );
-  mView->edit( mView->layerTreeModel()->node2index( newGroup ) );
+  mView->setCurrentIndex( mView->node2index( newGroup ) );
+  mView->edit( mView->node2index( newGroup ) );
 }
 
 void QgsLayerTreeViewDefaultActions::mutuallyExclusiveGroup()
