@@ -25,20 +25,16 @@
 // the widgets
 #include "qgssymbolslistwidget.h"
 #include "qgslayerpropertieswidget.h"
-#include "qgssymbollayerwidget.h"
-#include "qgsellipsesymbollayerwidget.h"
-#include "qgsvectorfieldsymbollayerwidget.h"
-
-#include "qgslogger.h"
 #include "qgsapplication.h"
-#include "qgssettings.h"
-#include "qgsfeatureiterator.h"
 #include "qgsvectorlayer.h"
 #include "qgssvgcache.h"
 #include "qgsimagecache.h"
 #include "qgsproject.h"
 #include "qgsguiutils.h"
 #include "qgsgui.h"
+#include "qgsmarkersymbol.h"
+#include "qgslinesymbol.h"
+#include "qscreen.h"
 
 #include <QColorDialog>
 #include <QPainter>
@@ -59,9 +55,9 @@ static const int SYMBOL_LAYER_ITEM_TYPE = QStandardItem::UserType + 1;
 DataDefinedRestorer::DataDefinedRestorer( QgsSymbol *symbol, const QgsSymbolLayer *symbolLayer )
 
 {
-  if ( symbolLayer->type() == QgsSymbol::Marker && symbol->type() == QgsSymbol::Marker )
+  if ( symbolLayer->type() == Qgis::SymbolType::Marker && symbol->type() == Qgis::SymbolType::Marker )
   {
-    Q_ASSERT( symbol->type() == QgsSymbol::Marker );
+    Q_ASSERT( symbol->type() == Qgis::SymbolType::Marker );
     mMarker = static_cast<QgsMarkerSymbol *>( symbol );
     mMarkerSymbolLayer = static_cast<const QgsMarkerSymbolLayer *>( symbolLayer );
     mDDSize = mMarker->dataDefinedSize();
@@ -70,7 +66,7 @@ DataDefinedRestorer::DataDefinedRestorer( QgsSymbol *symbol, const QgsSymbolLaye
     if ( !mDDSize && !mDDAngle )
       mMarker = nullptr;
   }
-  else if ( symbolLayer->type() == QgsSymbol::Line && symbol->type() == QgsSymbol::Line )
+  else if ( symbolLayer->type() == Qgis::SymbolType::Line && symbol->type() == Qgis::SymbolType::Line )
   {
     mLine = static_cast<QgsLineSymbol *>( symbol );
     mLineSymbolLayer = static_cast<const QgsLineSymbolLayer *>( symbolLayer );
@@ -122,21 +118,26 @@ void DataDefinedRestorer::restore()
 class SymbolLayerItem : public QStandardItem
 {
   public:
-    explicit SymbolLayerItem( QgsSymbolLayer *layer )
+    explicit SymbolLayerItem( QgsSymbolLayer *layer, Qgis::SymbolType symbolType, QgsVectorLayer *vectorLayer, QScreen *screen )
+      : mVectorLayer( vectorLayer )
+      , mScreen( screen )
     {
-      setLayer( layer );
+      setLayer( layer, symbolType );
     }
 
-    explicit SymbolLayerItem( QgsSymbol *symbol )
+    explicit SymbolLayerItem( QgsSymbol *symbol, QgsVectorLayer *vectorLayer, QScreen *screen )
+      : mVectorLayer( vectorLayer )
+      , mScreen( screen )
     {
       setSymbol( symbol );
     }
 
-    void setLayer( QgsSymbolLayer *layer )
+    void setLayer( QgsSymbolLayer *layer, Qgis::SymbolType symbolType )
     {
       mLayer = layer;
       mIsLayer = true;
       mSymbol = nullptr;
+      mSymbolType = symbolType;
       updatePreview();
     }
 
@@ -157,9 +158,13 @@ class SymbolLayerItem : public QStandardItem
       }
       QIcon icon;
       if ( mIsLayer )
-        icon = QgsSymbolLayerUtils::symbolLayerPreviewIcon( mLayer, QgsUnitTypes::RenderMillimeters, mSize ); //todo: make unit a parameter
+        icon = QgsSymbolLayerUtils::symbolLayerPreviewIcon( mLayer, Qgis::RenderUnit::Millimeters, mSize, QgsMapUnitScale(), mSymbol ? mSymbol->type() : mSymbolType, mVectorLayer, QgsScreenProperties( mScreen.data() ) );
       else
-        icon = QgsSymbolLayerUtils::symbolPreviewIcon( mSymbol, mSize );
+      {
+        QgsExpressionContext expContext;
+        expContext.appendScopes( QgsExpressionContextUtils::globalProjectLayerScopes( mVectorLayer ) );
+        icon = QIcon( QgsSymbolLayerUtils::symbolPreviewPixmap( mSymbol, mSize, 0, nullptr, false, &expContext, nullptr, QgsScreenProperties( mScreen.data() ) ) );
+      }
       setIcon( icon );
 
       if ( auto *lParent = parent() )
@@ -185,16 +190,22 @@ class SymbolLayerItem : public QStandardItem
       if ( role == Qt::DisplayRole || role == Qt::EditRole )
       {
         if ( mIsLayer )
-          return QgsApplication::symbolLayerRegistry()->symbolLayerMetadata( mLayer->layerType() )->visibleName();
+        {
+          QgsSymbolLayerAbstractMetadata *m = QgsApplication::symbolLayerRegistry()->symbolLayerMetadata( mLayer->layerType() );
+          if ( m )
+            return m->visibleName();
+          else
+            return QString();
+        }
         else
         {
           switch ( mSymbol->type() )
           {
-            case QgsSymbol::Marker :
+            case Qgis::SymbolType::Marker :
               return QCoreApplication::translate( "SymbolLayerItem", "Marker" );
-            case QgsSymbol::Fill   :
+            case Qgis::SymbolType::Fill   :
               return QCoreApplication::translate( "SymbolLayerItem", "Fill" );
-            case QgsSymbol::Line   :
+            case Qgis::SymbolType::Line   :
               return QCoreApplication::translate( "SymbolLayerItem", "Line" );
             default:
               return "Symbol";
@@ -203,12 +214,17 @@ class SymbolLayerItem : public QStandardItem
       }
       else if ( role == Qt::ForegroundRole && mIsLayer )
       {
-        QBrush brush( Qt::black, Qt::SolidPattern );
         if ( !mLayer->enabled() )
         {
-          brush.setColor( Qt::lightGray );
+          QPalette pal = qApp->palette();
+          QBrush brush = QStandardItem::data( role ).value< QBrush >();
+          brush.setColor( pal.color( QPalette::Disabled, QPalette::WindowText ) );
+          return brush;
+        }
+        else
+        {
+          return QVariant();
         }
-        return brush;
       }
 
 //      if ( role == Qt::SizeHintRole )
@@ -221,8 +237,11 @@ class SymbolLayerItem : public QStandardItem
   protected:
     QgsSymbolLayer *mLayer = nullptr;
     QgsSymbol *mSymbol = nullptr;
-    bool mIsLayer;
+    QPointer< QgsVectorLayer > mVectorLayer;
+    bool mIsLayer = false;
     QSize mSize;
+    Qgis::SymbolType mSymbolType = Qgis::SymbolType::Hybrid;
+    QPointer< QScreen > mScreen;
 };
 
 ///@endcond
@@ -254,7 +273,35 @@ QgsSymbolSelectorWidget::QgsSymbolSelectorWidget( QgsSymbol *symbol, QgsStyle *s
   iconLock.addFile( QgsApplication::iconPath( QStringLiteral( "locked.svg" ) ), QSize(), QIcon::Active, QIcon::On );
   iconLock.addFile( QgsApplication::iconPath( QStringLiteral( "unlocked.svg" ) ), QSize(), QIcon::Normal, QIcon::Off );
   iconLock.addFile( QgsApplication::iconPath( QStringLiteral( "unlocked.svg" ) ), QSize(), QIcon::Active, QIcon::Off );
-  btnLock->setIcon( iconLock );
+
+  QIcon iconColorLock;
+  iconColorLock.addFile( QgsApplication::iconPath( QStringLiteral( "mIconColorLocked.svg" ) ), QSize(), QIcon::Normal, QIcon::On );
+  iconColorLock.addFile( QgsApplication::iconPath( QStringLiteral( "mIconColorLocked.svg" ) ), QSize(), QIcon::Active, QIcon::On );
+  iconColorLock.addFile( QgsApplication::iconPath( QStringLiteral( "mIconColorUnlocked.svg" ) ), QSize(), QIcon::Normal, QIcon::Off );
+  iconColorLock.addFile( QgsApplication::iconPath( QStringLiteral( "mIconColorUnlocked.svg" ) ), QSize(), QIcon::Active, QIcon::Off );
+
+  mLockColorAction = new QAction( tr( "Lock Color" ), this );
+  mLockColorAction->setToolTip( tr( "Avoid changing the color of the layer when the symbol color is changed" ) );
+  mLockColorAction->setCheckable( true );
+  mLockColorAction->setIcon( iconColorLock );
+
+  QIcon iconSelectLock;
+  iconSelectLock.addFile( QgsApplication::iconPath( QStringLiteral( "mIconSelectLocked.svg" ) ), QSize(), QIcon::Normal, QIcon::On );
+  iconSelectLock.addFile( QgsApplication::iconPath( QStringLiteral( "mIconSelectLocked.svg" ) ), QSize(), QIcon::Active, QIcon::On );
+  iconSelectLock.addFile( QgsApplication::iconPath( QStringLiteral( "mIconSelectUnlocked.svg" ) ), QSize(), QIcon::Normal, QIcon::Off );
+  iconSelectLock.addFile( QgsApplication::iconPath( QStringLiteral( "mIconSelectUnlocked.svg" ) ), QSize(), QIcon::Active, QIcon::Off );
+
+  mLockSelectionColorAction = new QAction( tr( "Lock Color When Selected" ), this );
+  mLockSelectionColorAction->setToolTip( tr( "Avoid changing the color of the layer when a feature is selected" ) );
+  mLockSelectionColorAction->setCheckable( true );
+  mLockSelectionColorAction->setIcon( iconSelectLock );
+
+  QMenu *lockMenu = new QMenu( this );
+  lockMenu->addAction( mLockColorAction );
+  lockMenu->addAction( mLockSelectionColorAction );
+  btnLock->setMenu( lockMenu );
+  btnLock->setPopupMode( QToolButton::InstantPopup );
+
   btnDuplicate->setIcon( QIcon( QgsApplication::iconPath( "mActionDuplicateLayer.svg" ) ) );
   btnUp->setIcon( QIcon( QgsApplication::iconPath( "mActionArrowUp.svg" ) ) );
   btnDown->setIcon( QIcon( QgsApplication::iconPath( "mActionArrowDown.svg" ) ) );
@@ -292,14 +339,28 @@ QgsSymbolSelectorWidget::QgsSymbolSelectorWidget( QgsSymbol *symbol, QgsStyle *s
   connect( btnDown, &QAbstractButton::clicked, this, &QgsSymbolSelectorWidget::moveLayerDown );
   connect( btnAddLayer, &QAbstractButton::clicked, this, &QgsSymbolSelectorWidget::addLayer );
   connect( btnRemoveLayer, &QAbstractButton::clicked, this, &QgsSymbolSelectorWidget::removeLayer );
-  connect( btnLock, &QAbstractButton::clicked, this, &QgsSymbolSelectorWidget::lockLayer );
+  connect( mLockColorAction, &QAction::toggled, this, &QgsSymbolSelectorWidget::lockLayer );
+  connect( mLockSelectionColorAction, &QAction::toggled, this, [ = ]( bool checked )
+  {
+    QgsSymbolLayer *layer = currentLayer();
+    if ( !layer )
+      return;
+
+    Qgis::SymbolLayerUserFlags flags = layer->userFlags();
+    flags.setFlag( Qgis::SymbolLayerUserFlag::DisableSelectionRecoloring, checked );
+    layer->setUserFlags( flags );
+    updateLockButtonIcon();
+    emit symbolModified();
+  } );
   connect( btnDuplicate, &QAbstractButton::clicked, this, &QgsSymbolSelectorWidget::duplicateLayer );
   connect( this, &QgsSymbolSelectorWidget::symbolModified, this, &QgsPanelWidget::widgetChanged );
 
+  updateLockButtonIcon();
+
   updateUi();
 
   // set symbol as active item in the tree
-  QModelIndex newIndex = layersTree->model()->index( 0, 0 );
+  const QModelIndex newIndex = layersTree->model()->index( 0, 0 );
   layersTree->setCurrentIndex( newIndex );
 
   setPanelTitle( tr( "Symbol Selector" ) );
@@ -325,6 +386,14 @@ QgsSymbolSelectorWidget::QgsSymbolSelectorWidget( QgsSymbol *symbol, QgsStyle *s
   connect( QgsProject::instance(), static_cast < void ( QgsProject::* )( const QList<QgsMapLayer *>& layers ) > ( &QgsProject::layersWillBeRemoved ), this, &QgsSymbolSelectorWidget::layersAboutToBeRemoved );
 }
 
+QgsSymbolSelectorWidget *QgsSymbolSelectorWidget::createWidgetWithSymbolOwnership( std::unique_ptr<QgsSymbol> symbol, QgsStyle *style, QgsVectorLayer *vl, QWidget *parent )
+{
+  QgsSymbolSelectorWidget *widget = new QgsSymbolSelectorWidget( symbol.get(), style, vl, parent );
+  // transfer ownership of symbol to widget, so that we are guaranteed it will last for the duration of the widget
+  widget->mOwnedSymbol = std::move( symbol );
+  return widget;
+}
+
 QMenu *QgsSymbolSelectorWidget::advancedMenu()
 {
   if ( !mAdvancedMenu )
@@ -376,16 +445,16 @@ void QgsSymbolSelectorWidget::loadSymbol( QgsSymbol *symbol, SymbolLayerItem *pa
     parent = static_cast<SymbolLayerItem *>( mSymbolLayersModel->invisibleRootItem() );
   }
 
-  SymbolLayerItem *symbolItem = new SymbolLayerItem( symbol );
+  SymbolLayerItem *symbolItem = new SymbolLayerItem( symbol, mVectorLayer, screen() );
   QFont boldFont = symbolItem->font();
   boldFont.setBold( true );
   symbolItem->setFont( boldFont );
   parent->appendRow( symbolItem );
 
-  int count = symbol->symbolLayerCount();
+  const int count = symbol->symbolLayerCount();
   for ( int i = count - 1; i >= 0; i-- )
   {
-    SymbolLayerItem *layerItem = new SymbolLayerItem( symbol->symbolLayer( i ) );
+    SymbolLayerItem *layerItem = new SymbolLayerItem( symbol->symbolLayer( i ), symbol->type(), mVectorLayer, screen() );
     layerItem->setEditable( false );
     symbolItem->appendRow( layerItem );
     if ( symbol->symbolLayer( i )->subSymbol() )
@@ -411,7 +480,7 @@ void QgsSymbolSelectorWidget::reloadSymbol()
 
 void QgsSymbolSelectorWidget::updateUi()
 {
-  QModelIndex currentIdx = layersTree->currentIndex();
+  const QModelIndex currentIdx = layersTree->currentIndex();
   if ( !currentIdx.isValid() )
     return;
 
@@ -426,8 +495,8 @@ void QgsSymbolSelectorWidget::updateUi()
     return;
   }
 
-  int rowCount = item->parent()->rowCount();
-  int currentRow = item->row();
+  const int rowCount = item->parent()->rowCount();
+  const int currentRow = item->row();
 
   btnUp->setEnabled( currentRow > 0 );
   btnDown->setEnabled( currentRow < rowCount - 1 );
@@ -442,7 +511,7 @@ void QgsSymbolSelectorWidget::updatePreview()
     return;
 
   std::unique_ptr< QgsSymbol > symbolClone( mSymbol->clone() );
-  QImage preview = symbolClone->bigSymbolPreviewImage( &mPreviewExpressionContext );
+  const QImage preview = symbolClone->bigSymbolPreviewImage( &mPreviewExpressionContext, Qgis::SymbolPreviewFlag::FlagIncludeCrosshairsForMarkerSymbols, QgsScreenProperties( screen() ) );
   lblPreview->setPixmap( QPixmap::fromImage( preview ) );
   // Hope this is a appropriate place
   if ( !mBlockModified )
@@ -461,7 +530,7 @@ void QgsSymbolSelectorWidget::updateLayerPreview()
 
 SymbolLayerItem *QgsSymbolSelectorWidget::currentLayerItem()
 {
-  QModelIndex idx = layersTree->currentIndex();
+  const QModelIndex idx = layersTree->currentIndex();
   if ( !idx.isValid() )
     return nullptr;
 
@@ -474,7 +543,7 @@ SymbolLayerItem *QgsSymbolSelectorWidget::currentLayerItem()
 
 QgsSymbolLayer *QgsSymbolSelectorWidget::currentLayer()
 {
-  QModelIndex idx = layersTree->currentIndex();
+  const QModelIndex idx = layersTree->currentIndex();
   if ( !idx.isValid() )
     return nullptr;
 
@@ -546,7 +615,7 @@ void QgsSymbolSelectorWidget::symbolChanged()
   {
     //it is the symbol itself
     reloadSymbol();
-    QModelIndex newIndex = layersTree->model()->index( 0, 0 );
+    const QModelIndex newIndex = layersTree->model()->index( 0, 0 );
     layersTree->setCurrentIndex( newIndex );
   }
   updatePreview();
@@ -556,7 +625,7 @@ void QgsSymbolSelectorWidget::symbolChanged()
 
 void QgsSymbolSelectorWidget::setWidget( QWidget *widget )
 {
-  int index = stackedWidget->addWidget( widget );
+  const int index = stackedWidget->addWidget( widget );
   stackedWidget->setCurrentIndex( index );
   if ( mPresentWidget )
     mPresentWidget->deleteLater();
@@ -568,12 +637,27 @@ void QgsSymbolSelectorWidget::updateLockButton()
   QgsSymbolLayer *layer = currentLayer();
   if ( !layer )
     return;
-  btnLock->setChecked( layer->isLocked() );
+  mLockColorAction->setChecked( layer->isLocked() );
+  mLockSelectionColorAction->setChecked( layer->userFlags() & Qgis::SymbolLayerUserFlag::DisableSelectionRecoloring );
+
+  updateLockButtonIcon();
+}
+
+void QgsSymbolSelectorWidget::updateLockButtonIcon()
+{
+  if ( mLockColorAction->isChecked() && mLockSelectionColorAction->isChecked() )
+    btnLock->setIcon( QgsApplication::getThemeIcon( QStringLiteral( "locked.svg" ) ) );
+  else if ( mLockColorAction->isChecked() )
+    btnLock->setIcon( QgsApplication::getThemeIcon( QStringLiteral( "mIconColorLocked.svg" ) ) );
+  else if ( mLockSelectionColorAction->isChecked() )
+    btnLock->setIcon( QgsApplication::getThemeIcon( QStringLiteral( "mIconSelectLocked.svg" ) ) );
+  else
+    btnLock->setIcon( QgsApplication::getThemeIcon( QStringLiteral( "unlocked.svg" ) ) );
 }
 
 void QgsSymbolSelectorWidget::addLayer()
 {
-  QModelIndex idx = layersTree->currentIndex();
+  const QModelIndex idx = layersTree->currentIndex();
   if ( !idx.isValid() )
     return;
 
@@ -588,17 +672,17 @@ void QgsSymbolSelectorWidget::addLayer()
   QgsSymbol *parentSymbol = item->symbol();
 
   // save data-defined values at marker level
-  QgsProperty ddSize( parentSymbol->type() == QgsSymbol::Marker
-                      ? static_cast<QgsMarkerSymbol *>( parentSymbol )->dataDefinedSize()
-                      : QgsProperty() );
-  QgsProperty ddAngle( parentSymbol->type() == QgsSymbol::Marker
-                       ? static_cast<QgsMarkerSymbol *>( parentSymbol )->dataDefinedAngle()
-                       : QgsProperty() );
-  QgsProperty ddWidth( parentSymbol->type() == QgsSymbol::Line
-                       ? static_cast<QgsLineSymbol *>( parentSymbol )->dataDefinedWidth()
-                       : QgsProperty() );
+  const QgsProperty ddSize( parentSymbol->type() == Qgis::SymbolType::Marker
+                            ? static_cast<QgsMarkerSymbol *>( parentSymbol )->dataDefinedSize()
+                            : QgsProperty() );
+  const QgsProperty ddAngle( parentSymbol->type() == Qgis::SymbolType::Marker
+                             ? static_cast<QgsMarkerSymbol *>( parentSymbol )->dataDefinedAngle()
+                             : QgsProperty() );
+  const QgsProperty ddWidth( parentSymbol->type() == Qgis::SymbolType::Line
+                             ? static_cast<QgsLineSymbol *>( parentSymbol )->dataDefinedWidth()
+                             : QgsProperty() );
 
-  QgsSymbolLayer *newLayer = QgsApplication::symbolLayerRegistry()->defaultSymbolLayer( parentSymbol->type() );
+  QgsSymbolLayer *newLayer = QgsSymbolLayerRegistry::defaultSymbolLayer( parentSymbol->type() );
   if ( insertIdx == -1 )
     parentSymbol->appendSymbolLayer( newLayer );
   else
@@ -612,7 +696,7 @@ void QgsSymbolSelectorWidget::addLayer()
   if ( ddWidth )
     static_cast<QgsLineSymbol *>( parentSymbol )->setDataDefinedWidth( ddWidth );
 
-  SymbolLayerItem *newLayerItem = new SymbolLayerItem( newLayer );
+  SymbolLayerItem *newLayerItem = new SymbolLayerItem( newLayer, parentSymbol->type(), mVectorLayer, screen() );
   item->insertRow( insertIdx == -1 ? 0 : insertIdx, newLayerItem );
   item->updatePreview();
 
@@ -624,17 +708,17 @@ void QgsSymbolSelectorWidget::addLayer()
 void QgsSymbolSelectorWidget::removeLayer()
 {
   SymbolLayerItem *item = currentLayerItem();
-  int row = item->row();
+  const int row = item->row();
   SymbolLayerItem *parent = static_cast<SymbolLayerItem *>( item->parent() );
 
-  int layerIdx = parent->rowCount() - row - 1; // IMPORTANT
+  const int layerIdx = parent->rowCount() - row - 1; // IMPORTANT
   QgsSymbol *parentSymbol = parent->symbol();
   QgsSymbolLayer *tmpLayer = parentSymbol->takeSymbolLayer( layerIdx );
 
   parent->removeRow( row );
   parent->updatePreview();
 
-  QModelIndex newIdx = parent->child( 0 )->index();
+  const QModelIndex newIdx = parent->child( 0 )->index();
   layersTree->setCurrentIndex( newIdx );
 
   updateUi();
@@ -658,12 +742,12 @@ void QgsSymbolSelectorWidget::moveLayerByOffset( int offset )
   SymbolLayerItem *item = currentLayerItem();
   if ( !item )
     return;
-  int row = item->row();
+  const int row = item->row();
 
   SymbolLayerItem *parent = static_cast<SymbolLayerItem *>( item->parent() );
   QgsSymbol *parentSymbol = parent->symbol();
 
-  int layerIdx = parent->rowCount() - row - 1;
+  const int layerIdx = parent->rowCount() - row - 1;
   // switch layers
   QgsSymbolLayer *tmpLayer = parentSymbol->takeSymbolLayer( layerIdx );
   parentSymbol->insertSymbolLayer( layerIdx - offset, tmpLayer );
@@ -672,7 +756,7 @@ void QgsSymbolSelectorWidget::moveLayerByOffset( int offset )
   parent->insertRows( row + offset, rowItems );
   parent->updatePreview();
 
-  QModelIndex newIdx = rowItems[ 0 ]->index();
+  const QModelIndex newIdx = rowItems[ 0 ]->index();
   layersTree->setCurrentIndex( newIdx );
 
   updatePreview();
@@ -684,13 +768,14 @@ void QgsSymbolSelectorWidget::lockLayer()
   QgsSymbolLayer *layer = currentLayer();
   if ( !layer )
     return;
-  layer->setLocked( btnLock->isChecked() );
+  layer->setLocked( mLockColorAction->isChecked() );
+  updateLockButtonIcon();
   emit symbolModified();
 }
 
 void QgsSymbolSelectorWidget::duplicateLayer()
 {
-  QModelIndex idx = layersTree->currentIndex();
+  const QModelIndex idx = layersTree->currentIndex();
   if ( !idx.isValid() )
     return;
 
@@ -700,18 +785,19 @@ void QgsSymbolSelectorWidget::duplicateLayer()
 
   QgsSymbolLayer *source = item->layer();
 
-  int insertIdx = item->row();
+  const int insertIdx = item->row();
   item = static_cast<SymbolLayerItem *>( item->parent() );
 
   QgsSymbol *parentSymbol = item->symbol();
 
   QgsSymbolLayer *newLayer = source->clone();
+  QgsSymbolLayerUtils::resetSymbolLayerIds( newLayer );
   if ( insertIdx == -1 )
     parentSymbol->appendSymbolLayer( newLayer );
   else
     parentSymbol->insertSymbolLayer( item->rowCount() - insertIdx, newLayer );
 
-  SymbolLayerItem *newLayerItem = new SymbolLayerItem( newLayer );
+  SymbolLayerItem *newLayerItem = new SymbolLayerItem( newLayer, parentSymbol->type(), mVectorLayer, screen() );
   item->insertRow( insertIdx == -1 ? 0 : insertIdx, newLayerItem );
   if ( newLayer->subSymbol() )
   {
@@ -734,8 +820,10 @@ void QgsSymbolSelectorWidget::changeLayer( QgsSymbolLayer *newLayer )
   {
     item->removeRow( 0 );
   }
+  QgsSymbol *symbol = static_cast<SymbolLayerItem *>( item->parent() )->symbol();
+
   // update symbol layer item
-  item->setLayer( newLayer );
+  item->setLayer( newLayer, symbol->type() );
   // When it is a marker symbol
   if ( newLayer->subSymbol() )
   {
@@ -744,8 +832,7 @@ void QgsSymbolSelectorWidget::changeLayer( QgsSymbolLayer *newLayer )
   }
 
   // Change the symbol at last to avoid deleting item's layer
-  QgsSymbol *symbol = static_cast<SymbolLayerItem *>( item->parent() )->symbol();
-  int layerIdx = item->parent()->rowCount() - item->row() - 1;
+  const int layerIdx = item->parent()->rowCount() - item->row() - 1;
   symbol->changeSymbolLayer( layerIdx, newLayer );
 
   item->updatePreview();
@@ -773,7 +860,7 @@ QgsSymbolSelectorDialog::QgsSymbolSelectorDialog( QgsSymbol *symbol, QgsStyle *s
 
   mSelectorWidget->setMinimumSize( 460, 560 );
   setObjectName( QStringLiteral( "SymbolSelectorDialog" ) );
-  QgsGui::instance()->enableAutoGeometryRestore( this );
+  QgsGui::enableAutoGeometryRestore( this );
 
   // Can be embedded in renderer properties dialog
   if ( embedded )
