@@ -30,35 +30,33 @@
 #include "diagram/qgshistogramdiagram.h"
 #include "qgsdiagramrenderer.h"
 #include "qgsmaplayer.h"
-#include "qgsvectordataprovider.h"
 #include "qgsvectorlayer.h"
 #include "qgsapplication.h"
-#include "qgsproviderregistry.h"
 #include "qgsrenderer.h"
 #include "qgssinglesymbolrenderer.h"
 //qgis test includes
 #include "qgsmultirenderchecker.h"
-#include "qgspallabeling.h"
 #include "qgsproject.h"
 #include "qgsshadoweffect.h"
+#include "qgslinesymbol.h"
+#include "qgsmarkersymbol.h"
 
 /**
  * \ingroup UnitTests
  * Unit tests for the diagram renderer
  */
-class TestQgsDiagram : public QObject
+class TestQgsDiagram : public QgsTest
 {
     Q_OBJECT
 
   public:
-    TestQgsDiagram() = default;
+    TestQgsDiagram() : QgsTest( QStringLiteral( "Diagram Tests" ) ) {}
 
   private:
     bool mTestHasError =  false ;
     QgsMapSettings *mMapSettings = nullptr;
     QgsVectorLayer *mPointsLayer = nullptr;
     QString mTestDataDir;
-    QString mReport;
 
     bool imageCheck( const QString &testType );
 
@@ -78,19 +76,19 @@ class TestQgsDiagram : public QObject
       //
       //create a non spatial layer that will be used in all tests...
       //
-      QString myDataDir( TEST_DATA_DIR ); //defined in CmakeLists.txt
+      const QString myDataDir( TEST_DATA_DIR ); //defined in CmakeLists.txt
       mTestDataDir = myDataDir + '/';
 
       //
       //create a point layer that will be used in all tests...
       //
-      QString myPointsFileName = mTestDataDir + "points.shp";
-      QFileInfo myPointFileInfo( myPointsFileName );
+      const QString myPointsFileName = mTestDataDir + "points.shp";
+      const QFileInfo myPointFileInfo( myPointsFileName );
       mPointsLayer = new QgsVectorLayer( myPointFileInfo.filePath(),
                                          myPointFileInfo.completeBaseName(), QStringLiteral( "ogr" ) );
 
       //we don't want to render the points themselves, just the diagrams
-      QgsStringMap symbolProps;
+      QVariantMap symbolProps;
       symbolProps.insert( QStringLiteral( "color" ), QStringLiteral( "0,0,0,0" ) );
       symbolProps.insert( QStringLiteral( "outline_style" ), QStringLiteral( "no" ) );
       QgsMarkerSymbol *symbol = QgsMarkerSymbol::createSimple( symbolProps );
@@ -100,25 +98,14 @@ class TestQgsDiagram : public QObject
       // Create map composition to draw on
       QgsProject::instance()->addMapLayer( mPointsLayer );
       mMapSettings->setLayers( QList<QgsMapLayer *>() << mPointsLayer );
-
-      mReport += QLatin1String( "<h1>Diagram Tests</h1>\n" );
     }
 
     // will be called after the last testfunction was executed.
     void cleanupTestCase()
     {
       delete mMapSettings;
-      delete mPointsLayer;
+      QgsProject::instance()->removeAllMapLayers();
 
-      QString myReportFile = QDir::tempPath() + "/qgistest.html";
-      QFile myFile( myReportFile );
-      if ( myFile.open( QIODevice::WriteOnly | QIODevice::Append ) )
-      {
-        QTextStream myQTextStream( &myFile );
-        myQTextStream << mReport;
-        myFile.close();
-        //QDesktopServices::openUrl( "file:///" + myReportFile );
-      }
       QgsApplication::exitQgis();
     }
 
@@ -126,7 +113,7 @@ class TestQgsDiagram : public QObject
     void init()
     {
       mPointsLayer->setDiagramRenderer( nullptr );
-      QgsDiagramLayerSettings dls;
+      const QgsDiagramLayerSettings dls;
       mPointsLayer->setDiagramLayerSettings( dls );
     }
 
@@ -151,7 +138,7 @@ class TestQgsDiagram : public QObject
       ds.penColor = Qt::green;
       ds.penWidth = .5;
       ds.scaleByArea = true;
-      ds.sizeType = QgsUnitTypes::RenderMillimeters;
+      ds.sizeType = Qgis::RenderUnit::Millimeters;
       ds.size = QSizeF( 5, 5 );
       ds.rotationOffset = 0;
 
@@ -173,6 +160,117 @@ class TestQgsDiagram : public QObject
       QVERIFY( imageCheck( "piediagram" ) );
     }
 
+    void testPieDiagramOpacity()
+    {
+      QgsDiagramSettings ds;
+      QColor col1 = Qt::red;
+      QColor col2 = Qt::yellow;
+      col1.setAlphaF( 0.5 );
+      col2.setAlphaF( 1 );
+      ds.categoryColors = QList<QColor>() << col1 << col2;
+      ds.categoryAttributes = QList<QString>() << QStringLiteral( "\"Pilots\"" ) << QStringLiteral( "\"Cabin Crew\"" );
+      ds.minimumScale = -1;
+      ds.maximumScale = -1;
+      ds.minimumSize = 0;
+      ds.penColor = Qt::green;
+      ds.penWidth = .5;
+      ds.scaleByArea = true;
+      ds.sizeType = Qgis::RenderUnit::Millimeters;
+      ds.size = QSizeF( 5, 5 );
+      ds.rotationOffset = 0;
+
+      QgsLinearlyInterpolatedDiagramRenderer *dr = new QgsLinearlyInterpolatedDiagramRenderer();
+      dr->setLowerValue( 0.0 );
+      dr->setLowerSize( QSizeF( 0.0, 0.0 ) );
+      dr->setUpperValue( 10 );
+      dr->setUpperSize( QSizeF( 40, 40 ) );
+      dr->setClassificationField( QStringLiteral( "Staff" ) );
+      dr->setDiagram( new QgsPieDiagram() );
+      dr->setDiagramSettings( ds );
+      mPointsLayer->setDiagramRenderer( dr );
+
+      QgsDiagramLayerSettings dls = QgsDiagramLayerSettings();
+      dls.setPlacement( QgsDiagramLayerSettings::OverPoint );
+      dls.setShowAllDiagrams( true );
+      mPointsLayer->setDiagramLayerSettings( dls );
+
+      QVERIFY( imageCheck( "piediagram_opacity" ) );
+    }
+
+    void testPieDiagramAggregate()
+    {
+      QgsDiagramSettings ds;
+      QColor col1 = Qt::red;
+      QColor col2 = Qt::yellow;
+      col1.setAlphaF( 0.5 );
+      col2.setAlphaF( 0.5 );
+      ds.categoryColors = QList<QColor>() << col1 << col2;
+      ds.categoryAttributes = QList<QString>() << QStringLiteral( "\"Pilots\"/sum(\"Pilots\")" ) << QStringLiteral( "\"Cabin Crew\"/sum(\"Cabin Crew\")" );
+      ds.minimumScale = -1;
+      ds.maximumScale = -1;
+      ds.minimumSize = 0;
+      ds.penColor = Qt::green;
+      ds.penWidth = .5;
+      ds.scaleByArea = true;
+      ds.sizeType = Qgis::RenderUnit::Millimeters;
+      ds.size = QSizeF( 5, 5 );
+      ds.rotationOffset = 0;
+
+      QgsLinearlyInterpolatedDiagramRenderer *dr = new QgsLinearlyInterpolatedDiagramRenderer();
+      dr->setLowerValue( 0.0 );
+      dr->setLowerSize( QSizeF( 0.0, 0.0 ) );
+      dr->setUpperValue( 10 );
+      dr->setUpperSize( QSizeF( 40, 40 ) );
+      dr->setClassificationField( QStringLiteral( "Staff" ) );
+      dr->setDiagram( new QgsPieDiagram() );
+      dr->setDiagramSettings( ds );
+      mPointsLayer->setDiagramRenderer( dr );
+
+      QgsDiagramLayerSettings dls = QgsDiagramLayerSettings();
+      dls.setPlacement( QgsDiagramLayerSettings::OverPoint );
+      dls.setShowAllDiagrams( true );
+      mPointsLayer->setDiagramLayerSettings( dls );
+
+      QVERIFY( imageCheck( "piediagram_aggregate" ) );
+    }
+
+    void testDiagramWithGeometryBasedExpressionAttribute()
+    {
+      QgsDiagramSettings ds;
+      QColor col1 = Qt::red;
+      QColor col2 = Qt::yellow;
+      col1.setAlphaF( 0.5 );
+      col2.setAlphaF( 0.5 );
+      ds.categoryColors = QList<QColor>() << col1 << col2;
+      ds.categoryAttributes = QList<QString>() << QStringLiteral( "abs($x)" ) << QStringLiteral( "$y" );
+      ds.minimumScale = -1;
+      ds.maximumScale = -1;
+      ds.minimumSize = 0;
+      ds.penColor = Qt::green;
+      ds.penWidth = .5;
+      ds.scaleByArea = true;
+      ds.sizeType = Qgis::RenderUnit::Millimeters;
+      ds.size = QSizeF( 5, 5 );
+      ds.rotationOffset = 0;
+
+      QgsLinearlyInterpolatedDiagramRenderer *dr = new QgsLinearlyInterpolatedDiagramRenderer();
+      dr->setLowerValue( 0.0 );
+      dr->setLowerSize( QSizeF( 0.0, 0.0 ) );
+      dr->setUpperValue( 10 );
+      dr->setUpperSize( QSizeF( 40, 40 ) );
+      dr->setClassificationField( QStringLiteral( "Staff" ) );
+      dr->setDiagram( new QgsPieDiagram() );
+      dr->setDiagramSettings( ds );
+      mPointsLayer->setDiagramRenderer( dr );
+
+      QgsDiagramLayerSettings dls = QgsDiagramLayerSettings();
+      dls.setPlacement( QgsDiagramLayerSettings::OverPoint );
+      dls.setShowAllDiagrams( true );
+      mPointsLayer->setDiagramLayerSettings( dls );
+
+      QVERIFY( imageCheck( "piediagram_geometry_based_expression" ) );
+    }
+
     void testPaintEffect()
     {
       QgsDiagramSettings ds;
@@ -188,7 +286,7 @@ class TestQgsDiagram : public QObject
       ds.penColor = Qt::green;
       ds.penWidth = .5;
       ds.scaleByArea = true;
-      ds.sizeType = QgsUnitTypes::RenderMillimeters;
+      ds.sizeType = Qgis::RenderUnit::Millimeters;
       ds.size = QSizeF( 5, 5 );
       ds.rotationOffset = 0;
       ds.setPaintEffect( new QgsDropShadowEffect() );
@@ -226,7 +324,7 @@ class TestQgsDiagram : public QObject
       ds.penColor = Qt::green;
       ds.penWidth = .5;
       ds.scaleByArea = true;
-      ds.sizeType = QgsUnitTypes::RenderMillimeters;
+      ds.sizeType = Qgis::RenderUnit::Millimeters;
       ds.size = QSizeF( 5, 5 );
       ds.rotationOffset = 0;
 
@@ -263,11 +361,11 @@ class TestQgsDiagram : public QObject
       ds.penColor = Qt::green;
       ds.penWidth = .5;
       ds.scaleByArea = true;
-      ds.sizeType = QgsUnitTypes::RenderMillimeters;
+      ds.sizeType = Qgis::RenderUnit::Millimeters;
       ds.size = QSizeF( 5, 5 );
       ds.rotationOffset = 0;
       ds.setSpacing( 17 );
-      ds.setSpacingUnit( QgsUnitTypes::RenderPoints );
+      ds.setSpacingUnit( Qgis::RenderUnit::Points );
 
       QgsLinearlyInterpolatedDiagramRenderer *dr = new QgsLinearlyInterpolatedDiagramRenderer();
       dr->setLowerValue( 0.0 );
@@ -302,12 +400,12 @@ class TestQgsDiagram : public QObject
       ds.penColor = Qt::green;
       ds.penWidth = .5;
       ds.scaleByArea = true;
-      ds.sizeType = QgsUnitTypes::RenderMillimeters;
+      ds.sizeType = Qgis::RenderUnit::Millimeters;
       ds.size = QSizeF( 5, 5 );
       ds.rotationOffset = 0;
       ds.setShowAxis( true );
 
-      QgsStringMap props;
+      QVariantMap props;
       props.insert( QStringLiteral( "width" ), QStringLiteral( "2" ) );
       props.insert( QStringLiteral( "color" ), QStringLiteral( "#ff00ff" ) );
       ds.setAxisLineSymbol( QgsLineSymbol::createSimple( props ) );
@@ -357,7 +455,7 @@ class TestQgsDiagram : public QObject
       ds.penColor = Qt::green;
       ds.penWidth = .5;
       ds.scaleByArea = true;
-      ds.sizeType = QgsUnitTypes::RenderMillimeters;
+      ds.sizeType = Qgis::RenderUnit::Millimeters;
       ds.size = QSizeF( 5, 5 );
       ds.rotationOffset = 0;
       ds.diagramOrientation = QgsDiagramSettings::Right;
@@ -404,7 +502,7 @@ class TestQgsDiagram : public QObject
       ds.penColor = Qt::green;
       ds.penWidth = .5;
       ds.scaleByArea = true;
-      ds.sizeType = QgsUnitTypes::RenderMillimeters;
+      ds.sizeType = Qgis::RenderUnit::Millimeters;
       ds.size = QSizeF( 15, 15 );
       ds.rotationOffset = 0;
       ds.diagramOrientation = QgsDiagramSettings::Up;
@@ -450,7 +548,7 @@ class TestQgsDiagram : public QObject
       ds.penColor = Qt::green;
       ds.penWidth = .5;
       ds.scaleByArea = true;
-      ds.sizeType = QgsUnitTypes::RenderMillimeters;
+      ds.sizeType = Qgis::RenderUnit::Millimeters;
       ds.size = QSizeF( 5, 5 );
       ds.rotationOffset = 0;
       ds.diagramOrientation = QgsDiagramSettings::Up;
@@ -501,13 +599,13 @@ class TestQgsDiagram : public QObject
       ds.penColor = Qt::green;
       ds.penWidth = .5;
       ds.scaleByArea = true;
-      ds.sizeType = QgsUnitTypes::RenderMillimeters;
+      ds.sizeType = Qgis::RenderUnit::Millimeters;
       ds.size = QSizeF( 5, 5 );
       ds.rotationOffset = 0;
       ds.setSpacing( 3 );
       ds.setShowAxis( true );
 
-      QgsStringMap props;
+      QVariantMap props;
       props.insert( QStringLiteral( "width" ), QStringLiteral( "2" ) );
       props.insert( QStringLiteral( "color" ), QStringLiteral( "#ff00ff" ) );
       ds.setAxisLineSymbol( QgsLineSymbol::createSimple( props ) );
@@ -557,13 +655,13 @@ class TestQgsDiagram : public QObject
       ds.penColor = Qt::green;
       ds.penWidth = .5;
       ds.scaleByArea = true;
-      ds.sizeType = QgsUnitTypes::RenderMillimeters;
+      ds.sizeType = Qgis::RenderUnit::Millimeters;
       ds.size = QSizeF( 5, 5 );
       ds.rotationOffset = 0;
       ds.setSpacing( 3 );
       ds.setShowAxis( true );
 
-      QgsStringMap props;
+      QVariantMap props;
       props.insert( QStringLiteral( "width" ), QStringLiteral( "2" ) );
       props.insert( QStringLiteral( "color" ), QStringLiteral( "#ff00ff" ) );
       ds.setAxisLineSymbol( QgsLineSymbol::createSimple( props ) );
@@ -613,7 +711,7 @@ class TestQgsDiagram : public QObject
       ds.penColor = Qt::green;
       ds.penWidth = .5;
       ds.scaleByArea = true;
-      ds.sizeType = QgsUnitTypes::RenderMillimeters;
+      ds.sizeType = Qgis::RenderUnit::Millimeters;
       ds.size = QSizeF( 5, 5 );
       ds.rotationOffset = 0;
 
@@ -655,7 +753,7 @@ class TestQgsDiagram : public QObject
       ds.penColor = Qt::green;
       ds.penWidth = .5;
       ds.scaleByArea = true;
-      ds.sizeType = QgsUnitTypes::RenderMillimeters;
+      ds.sizeType = Qgis::RenderUnit::Millimeters;
       ds.size = QSizeF( 15, 15 );
       ds.rotationOffset = 90;
       ds.setDirection( QgsDiagramSettings::Clockwise );
@@ -692,7 +790,7 @@ class TestQgsDiagram : public QObject
       ds.penColor = Qt::green;
       ds.penWidth = .5;
       ds.scaleByArea = true;
-      ds.sizeType = QgsUnitTypes::RenderMillimeters;
+      ds.sizeType = Qgis::RenderUnit::Millimeters;
       ds.size = QSizeF( 15, 15 );
       ds.rotationOffset = 0;
 
@@ -729,7 +827,7 @@ class TestQgsDiagram : public QObject
       ds.penColor = Qt::green;
       ds.penWidth = .5;
       ds.scaleByArea = true;
-      ds.sizeType = QgsUnitTypes::RenderMillimeters;
+      ds.sizeType = Qgis::RenderUnit::Millimeters;
       ds.size = QSizeF( 15, 15 );
       ds.rotationOffset = 0;
 
@@ -766,7 +864,7 @@ class TestQgsDiagram : public QObject
       ds.penColor = Qt::green;
       ds.penWidth = .5;
       ds.scaleByArea = true;
-      ds.sizeType = QgsUnitTypes::RenderMillimeters;
+      ds.sizeType = Qgis::RenderUnit::Millimeters;
       ds.size = QSizeF( 15, 15 );
       ds.rotationOffset = 0;
 
@@ -802,7 +900,7 @@ class TestQgsDiagram : public QObject
       ds.penColor = Qt::green;
       ds.penWidth = .5;
       ds.scaleByArea = true;
-      ds.sizeType = QgsUnitTypes::RenderMillimeters;
+      ds.sizeType = Qgis::RenderUnit::Millimeters;
       ds.size = QSizeF( 15, 15 );
       ds.rotationOffset = 0;
 
@@ -838,7 +936,7 @@ class TestQgsDiagram : public QObject
       ds.penColor = Qt::green;
       ds.penWidth = .5;
       ds.scaleByArea = true;
-      ds.sizeType = QgsUnitTypes::RenderMillimeters;
+      ds.sizeType = Qgis::RenderUnit::Millimeters;
       ds.size = QSizeF( 15, 15 );
       ds.rotationOffset = 0;
 
@@ -874,7 +972,7 @@ class TestQgsDiagram : public QObject
       ds.penColor = Qt::green;
       ds.penWidth = .5;
       ds.scaleByArea = true;
-      ds.sizeType = QgsUnitTypes::RenderMillimeters;
+      ds.sizeType = Qgis::RenderUnit::Millimeters;
       ds.size = QSizeF( 50, 50 );
       ds.rotationOffset = 0;
 
@@ -898,8 +996,8 @@ class TestQgsDiagram : public QObject
     void testDataDefinedZIndex()
     {
       QgsDiagramSettings ds;
-      QColor col1 = Qt::red;
-      QColor col2 = Qt::yellow;
+      const QColor col1 = Qt::red;
+      const QColor col2 = Qt::yellow;
       ds.categoryColors = QList<QColor>() << col1 << col2;
       ds.categoryAttributes = QList<QString>() << QStringLiteral( "\"Pilots\"" ) << QStringLiteral( "\"Cabin Crew\"" );
       ds.minimumScale = -1;
@@ -908,7 +1006,7 @@ class TestQgsDiagram : public QObject
       ds.penColor = Qt::green;
       ds.penWidth = .5;
       ds.scaleByArea = true;
-      ds.sizeType = QgsUnitTypes::RenderMillimeters;
+      ds.sizeType = Qgis::RenderUnit::Millimeters;
       ds.size = QSizeF( 50, 50 );
       ds.rotationOffset = 0;
 
@@ -944,7 +1042,7 @@ class TestQgsDiagram : public QObject
       ds.penColor = Qt::green;
       ds.penWidth = .5;
       ds.scaleByArea = true;
-      ds.sizeType = QgsUnitTypes::RenderMillimeters;
+      ds.sizeType = Qgis::RenderUnit::Millimeters;
       ds.size = QSizeF( 50, 50 );
       ds.rotationOffset = 0;
 
@@ -985,7 +1083,7 @@ class TestQgsDiagram : public QObject
       ds.penColor = Qt::green;
       ds.penWidth = .5;
       ds.scaleByArea = true;
-      ds.sizeType = QgsUnitTypes::RenderMillimeters;
+      ds.sizeType = Qgis::RenderUnit::Millimeters;
       ds.size = QSizeF( 15, 15 );
       ds.rotationOffset = 0;
 
@@ -1011,7 +1109,7 @@ class TestQgsDiagram : public QObject
       const QString filename = QStringLiteral( TEST_DATA_DIR ) + "/lines.shp";
       std::unique_ptr< QgsVectorLayer> vl2( new QgsVectorLayer( filename, QStringLiteral( "lines" ), QStringLiteral( "ogr" ) ) );
 
-      QgsStringMap props;
+      QVariantMap props;
       props.insert( QStringLiteral( "outline_color" ), QStringLiteral( "#487bb6" ) );
       props.insert( QStringLiteral( "outline_width" ), QStringLiteral( "1" ) );
       std::unique_ptr< QgsLineSymbol > symbol( QgsLineSymbol::createSimple( props ) );
@@ -1030,7 +1128,7 @@ class TestQgsDiagram : public QObject
       ds.penColor = Qt::green;
       ds.penWidth = .5;
       ds.scaleByArea = true;
-      ds.sizeType = QgsUnitTypes::RenderMillimeters;
+      ds.sizeType = Qgis::RenderUnit::Millimeters;
       ds.size = QSizeF( 5, 5 );
       ds.rotationOffset = 0;
 
@@ -1070,16 +1168,16 @@ bool TestQgsDiagram::imageCheck( const QString &testType )
   //use the QgsRenderChecker test utility class to
   //ensure the rendered output matches our control image
 
-  QgsRectangle extent( -126, 23, -70, 47 );
+  const QgsRectangle extent( -126, 23, -70, 47 );
   mMapSettings->setExtent( extent );
-  mMapSettings->setFlag( QgsMapSettings::ForceVectorOutput );
+  mMapSettings->setFlag( Qgis::MapSettingsFlag::ForceVectorOutput );
   mMapSettings->setOutputDpi( 96 );
   QgsMultiRenderChecker checker;
   checker.setControlPathPrefix( QStringLiteral( "diagrams" ) );
   checker.setControlName( "expected_" + testType );
   checker.setMapSettings( *mMapSettings );
   checker.setColorTolerance( 15 );
-  bool resultFlag = checker.runTest( testType, 200 );
+  const bool resultFlag = checker.runTest( testType, 200 );
   mReport += checker.report();
   return resultFlag;
 }
